#! /usr/bin/perl 
#
#
# strfile.pl - .dat file creator for AQOTDD.
#
# Copyright (C) 2000 Afghan Hound Software
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use Getopt::Std;
use vars qw/ $opt_h $opt_v $opt_i $opt_s $opt_r /;
use vars qw/ $opt_x $opt_q $opt_c $opt_f $opt_o /;

my (@quotes,@offsets,@idx);
my $version  = 1;    # Does this ever change?
my $min_len  = 1e17; # :-)
my $max_len  = 0;
my $strflags = 0;
my $delmntng = '%';  # By default.

sub print_version
  {
    print "Afghan strfile.pl version 1.0.0. Copyright (C) 2000 Afghan Hound.\n\n";
    print "  *** This program comes with ABSOLUTELY NO WARRANTY. *** \n\n";
    print "You may redistribute copies of Afghan fortune.pl under the terms\n";
    print "of the GNU General Public License. For more information about\n";
    print "these matters, see the file named COPYING.\n";
    exit(0);
  }

sub print_help
  {
    print "Afghan strfile.pl version 1.0.0. Copyright (C) 2000 Afghan Hound.\n";
    print "Usage:\n";
    print "     -f infile   : Input filename.\n";
    print "     -o outfile  : Output filename (defaults to infile.dat).\n";
    print "     -c char     : File deliminating character. (default '%')\n";
    print "     -x          : Mark infile as being rot13 encoded.\n";
    print "     -r          : Randomise the file offsets.\n";
    print "     -s          : Sort the offsets alphabetically.\n";
    print "     -i          : (With -s) Ignore case.\n";
    print "     -q          : Do not print stats on exit.\n";
    print "     -h          : print this help.\n";	
    print "     -v          : print version information.\n";
    print "\nNote that the default ordering is the order present in infile, and\n";
    print "the '-r' flag overrides the '-s' and/or '-i' flags.\n\n";
    exit(0);
  }

# fisher_yates_shuffle( \@array ) : 
# generate a random permutation of @array in place
# 
# Straight out of the docs :)
sub fisher_yates_shuffle 
  {
    my $array = shift;
    my ($i,$j);
    for ($i = @$array; --$i; ) 
      {
	$j = int rand ($i+1);
	next if $i == $j;
	@$array[$i,$j] = @$array[$j,$i];
      }
  }

sub print_stats
  {
    print "Input File  :",$opt_f,"\n";
    print "Total Quotes:",(1+$#quotes),"\n";
    print "Shortest is :",$min_len,"\n";
    print "Longest is  :",$max_len,"\n";
    print "Flags       :";
    print "random=",(($strflags)&(0x1)),", ";
    print "sorted=",((($strflags)&(0x2))/(0x2));
    print "(nocase=",(($opt_i)&(0x1)),"), ";
    print "rot13=",((($strflags)&(0x4))/(0x4)),"\n";
    print "Output File :",$opt_o,"\n"; 
  }

sub write_file
  {
    my ($file,$tmp);
    $file = ">".$opt_o;
    open (FILE,$file) or die("Can't open $opt_o!");
    $tmp = pack("NNNNN",$version,(1+$#quotes),$max_len,$min_len,$strflags);
    print FILE $tmp,$delmntng,"\0\0\0"; #pack to end of word;
    for (@offsets)
      {
	$tmp = pack("N",$_);
	print FILE $tmp;
      }
    close(FILE);
  }

sub get_quotes
  {
  my ($filename,$delim) = @_;
  my ($i,$j,$len,$line,$offset,$quote);
  open (FILE,$filename) or die("Can't open $filename!");
  $i      = 0;
  $offset = 0;
  $quote  = '';
  while( <FILE>)
    {
      $line=$_;
      if ($line eq "%\n")
	{
	  $len = length($quote);
	  ($len > $max_len)&&($max_len = $len);
	  ($len)&&($len < $min_len)&&($min_len = $len);
	  if ($len)
	    {
	      $quotes[$i] = $quote;
	      $quote = '';
	      $offsets[$i] = $offset;
	      $offset += ($len+2);
	      $i++;
	    }
	}
      else 
	{
	  $quote.=$line;
	}
    }
  $offsets[$i] = $offset;
  close(FILE);
}

getopts('hvisrxqc:f:o:');

($opt_h)&&(print_help());
($opt_v)&&(print_version());
($opt_c)&&($delmntng = $opt_c);
($opt_o)||($opt_o=$opt_f.'.dat');
get_quotes($opt_f,$delmntng);


if ($opt_s)
  {
    $strflags = 2;
    if ($opt_i)
      {
	@offsets=@offsets[sort{uc($quotes[$a]) cmp uc($quotes[$b])} 0 .. $#quotes];
      }
    else
      {
	@offsets=@offsets[sort{$quotes[$a] cmp $quotes[$b]} 0 .. $#quotes];
      }
  }

($opt_r)&&(fisher_yates_shuffle(\@offsets))&&($strflags = 1);
($opt_x)&&($strflags += 4);
($opt_q)||(print_stats());

write_file();
exit(0);
